#region Copyright (C) 2003-2010 Stimulsoft
/*
{*******************************************************************}
{																	}
{	Stimulsoft Reports       										}
{																	}
{	Copyright (C) 2003-2010 Stimulsoft     							}
{	ALL RIGHTS RESERVED												}
{																	}
{	The entire contents of this file is protected by U.S. and		}
{	International Copyright Laws. Unauthorized reproduction,		}
{	reverse-engineering, and distribution of all or any portion of	}
{	the code contained in this file is strictly prohibited and may	}
{	result in severe civil and criminal penalties and will be		}
{	prosecuted to the maximum extent possible under the law.		}
{																	}
{	RESTRICTIONS													}
{																	}
{	THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES			}
{	ARE CONFIDENTIAL AND PROPRIETARY								}
{	TRADE SECRETS OF Stimulsoft										}
{																	}
{	CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON		}
{	ADDITIONAL RESTRICTIONS.										}
{																	}
{*******************************************************************}
*/
#endregion Copyright (C) 2003-2010 Stimulsoft

using System;
using System.Collections;
using System.Data;
using System.ComponentModel;
using Devart.Data.MySql;
using Stimulsoft.Base;
using Stimulsoft.Base.Localization;
using Stimulsoft.Base.Serializing;
using Stimulsoft.Report.Dictionary.Design;
using Stimulsoft.Report.Events;


namespace Stimulsoft.Report.Dictionary
{
	[TypeConverter(typeof(StiSqlSourceConverter))]
	public class StiMySqlSource : StiSqlSource
	{
		#region DataAdapter
		protected override Type ConvertDbTypeToTypeInternal(int sqlType)
		{
            MySqlType dbType = (MySqlType)sqlType;
			switch (dbType)
			{
                case MySqlType.BigInt:
                case MySqlType.Int:
                case MySqlType.SmallInt:
                case MySqlType.TinyInt:
					return typeof(Int64);

                case MySqlType.Decimal:
					return typeof(decimal);

                case MySqlType.Double:
                case MySqlType.Float:
					return typeof(double);
    
                case MySqlType.Date:
                case MySqlType.DateTime:
                //case MySqlType.Newdate:
					return typeof(DateTime);

				default:
					return typeof(string);
			}
		}

		public override Type GetParameterTypesEnum()
		{
        return typeof(MySqlType);
		}

		public override StiDataParameter AddParameter()
		{
			StiDataParameter parameter = new StiDataParameter(
            StiLocalization.Get("PropertyMain", "Parameter"), string.Empty, (int)MySqlType.VarChar, 0);
			Parameters.Add(parameter);
			return parameter;
		}

        public virtual StiDataParameter AddParameter(string name, string expression, MySqlType type, int size)
		{
			StiDataParameter parameter = AddParameter();
			parameter.Name = name;
			parameter.Expression = expression;
			parameter.Type = (int)type;
			parameter.Size = size;

			return parameter;
		}

		public override void UpdateParameters()
		{
			if (this.DataTable != null)
			{
				InvokeConnecting();
				foreach (StiDataParameter parameter in Parameters)
				{
					((MySqlDataAdapter)DataAdapter).SelectCommand.Parameters[parameter.Name].Value = parameter.ParameterValue;
				}
				DataTable dataTable = this.DataTable;
				dataTable.Rows.Clear();

                ((MySqlDataAdapter)DataAdapter).SelectCommand.CommandTimeout = this.CommandTimeout;
                ((MySqlDataAdapter)DataAdapter).Fill(dataTable);
			
				CheckColumnsIndexs();
			}			
		}

		protected override string DataAdapterType
		{
			get
			{
				return "Stimulsoft.Report.Dictionary.StiMySqlAdapterService";
			}
		}
		#endregion

		#region this
		public StiMySqlSource() : this("", "", "")
		{
		}

		public StiMySqlSource(string nameInSource, string name) : this(nameInSource, name, name)
		{
		}

		public StiMySqlSource(string nameInSource, string name, string alias) : this(nameInSource, name, alias, string.Empty)
		{

		}

		public StiMySqlSource(string nameInSource, string name, string alias, string sqlCommand) : 
			base(nameInSource, name, alias, sqlCommand)			
		{
		}

		public StiMySqlSource(string nameInSource, string name, string alias, string sqlCommand, 
			bool connectOnStart) : 
		base(nameInSource, name, alias, sqlCommand, connectOnStart)
		{
		}

		public StiMySqlSource(string nameInSource, string name, string alias, string sqlCommand, 
			bool connectOnStart, bool reconnectOnEachRow) : 
			base(nameInSource, name, alias, sqlCommand, connectOnStart, reconnectOnEachRow)
		{
		}

       public StiMySqlSource(string nameInSource, string name, string alias, string sqlCommand, 
			bool connectOnStart, bool reconnectOnEachRow, int commandTimeout, string key) : 
			base(nameInSource, name, alias, sqlCommand, connectOnStart, reconnectOnEachRow, commandTimeout, key)
		{
		}
		#endregion
	}
}

