#region Copyright (C) 2003-2015 Stimulsoft
/*
{*******************************************************************}
{																	}
{	Stimulsoft Reports       										}
{																	}
{	Copyright (C) 2003-2015 Stimulsoft     							}
{	ALL RIGHTS RESERVED												}
{																	}
{	The entire contents of this file is protected by U.S. and		}
{	International Copyright Laws. Unauthorized reproduction,		}
{	reverse-engineering, and distribution of all or any portion of	}
{	the code contained in this file is strictly prohibited and may	}
{	result in severe civil and criminal penalties and will be		}
{	prosecuted to the maximum extent possible under the law.		}
{																	}
{	RESTRICTIONS													}
{																	}
{	THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES			}
{	ARE CONFIDENTIAL AND PROPRIETARY								}
{	TRADE SECRETS OF Stimulsoft										}
{																	}
{	CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON		}
{	ADDITIONAL RESTRICTIONS.										}
{																	}
{*******************************************************************}
*/
#endregion Copyright (C) 2003-2015 Stimulsoft

using System;
using System.Data;
using System.ComponentModel;
using System.Collections;
using System.Windows.Forms;
using Stimulsoft.Report.Dictionary.Design;
using Stimulsoft.Base.Localization;
using MySql.Data.MySqlClient;

namespace Stimulsoft.Report.Dictionary
{
    [TypeConverter(typeof(StiSqlDatabaseConverter))]
    public class StiMySqlDatabase : StiSqlDatabase
    {
        #region StiService override
        public override string ServiceName
        {
            get
            {
                return StiLocalization.Get("Database", "DatabaseMySQL");
            }
        }
        #endregion

        #region DataAdapter override
        protected override string DataAdapterType
        {
            get
            {
                return "Stimulsoft.Report.Dictionary.StiMySqlAdapterService";
            }
        }
        #endregion


        #region Methods
        /// <summary>
        /// Adds tables, views and stored procedures to report dictionary from database information.
        /// </summary>
        public override void ApplyDatabaseInformation(StiDatabaseInformation information, StiReport report, StiDatabaseInformation informationAll)
        {
            #region Tables

            foreach (DataTable dataTable in information.Tables)
            {
                StiMySqlSource source = new StiMySqlSource(this.Name,
                    StiNameCreation.CreateName(report, dataTable.TableName, false, false, true));
                string table = dataTable.TableName;
                if (table.Trim().Contains(" ")) table = string.Format("[{0}]", table);
                source.SqlCommand = "select * from " + table;

                foreach (DataColumn dataColumn in dataTable.Columns)
                {
                    StiDataColumn column = new StiDataColumn(dataColumn.ColumnName, dataColumn.DataType);
                    source.Columns.Add(column);
                }
                report.Dictionary.DataSources.Add(source);
            }
            #endregion

            #region Views
            foreach (DataTable dataTable in information.Views)
            {
                StiMySqlSource source = new StiMySqlSource(this.Name,
                    StiNameCreation.CreateName(report, dataTable.TableName, false, false, true));
                string table = dataTable.TableName;
                if (table.Trim().Contains(" ")) table = string.Format("[{0}]", table);
                source.SqlCommand = "select * from " + table;

                foreach (DataColumn dataColumn in dataTable.Columns)
                {
                    StiDataColumn column = new StiDataColumn(dataColumn.ColumnName, dataColumn.DataType);
                    source.Columns.Add(column);
                }
                report.Dictionary.DataSources.Add(source);
            }
            #endregion

            #region StoredProcedures
            foreach (DataTable dataTable in information.StoredProcedures)
            {
                StiMySqlSource source = new StiMySqlSource(this.Name,
                    StiNameCreation.CreateName(report, dataTable.TableName, false, false, true));
                source.SqlCommand = "call " + dataTable.TableName;
                string paramString = "";

                foreach (DataColumn dataColumn in dataTable.Columns)
                {
                    if (dataColumn.Caption == "Columns")
                    {
                        StiDataColumn procedureColumn = new StiDataColumn(dataColumn.ColumnName, dataColumn.DataType);
                        procedureColumn.Name = dataColumn.ColumnName;
                        source.Columns.Add(procedureColumn);
                    }
                    else
                    {
                        StiDataParameter procedureParameter = new StiDataParameter();
                        procedureParameter.Name = dataColumn.ColumnName;
                        source.Parameters.Add(procedureParameter);

                        paramString += dataColumn.ColumnName + ",";
                    }
                }
                if (source.Parameters.Count > 0)
                    source.Type = StiSqlSourceType.StoredProcedure;

                source.SqlCommand += "(" + paramString.Substring(0, paramString.Length - 1) + ")";

                report.Dictionary.DataSources.Add(source);
            }
            #endregion
        }

        public override void ApplyDatabaseInformation(StiDatabaseInformation information, StiReport report)
        {
            ApplyDatabaseInformation(information, report, null);
        }

        /// <summary>
        /// Returns full database information.
        /// </summary>
        public override StiDatabaseInformation GetDatabaseInformation()
        {
            StiDatabaseInformation information = new StiDatabaseInformation();
            try
            {
                using (MySqlConnection connection = new MySqlConnection(this.ConnectionString))
                {
                    connection.Open();

                    #region Tables
                    Hashtable tableHash = new Hashtable();
                    try
                    {
                        DataTable tables = connection.GetSchema("Tables");

                        foreach (DataRow row in tables.Rows)
                        {
                            if (row["TABLE_SCHEMA"] != DBNull.Value &&
                                ((string)row["TABLE_SCHEMA"]) == "sys") continue;

                            DataTable table = new DataTable(row["TABLE_NAME"] as string);

                            if (row["TABLE_TYPE"] != DBNull.Value &&
                                (((string)row["TABLE_SCHEMA"] == connection.Database) && ((string)row["TABLE_TYPE"] == "BASE TABLE")))
                            {
                                tableHash[table.TableName] = table;
                                information.Tables.Add(table);
                            }
                        }
                    }
                    catch
                    {
                    }
                    #endregion

                    #region Views
                    Hashtable viewHash = new Hashtable();
                    try
                    {
                        DataTable views = connection.GetSchema("Views");

                        foreach (DataRow row in views.Rows)
                        {
                            if (row["TABLE_SCHEMA"] != DBNull.Value &&
                                ((string)row["TABLE_SCHEMA"]) != connection.Database) continue;

                            DataTable view = new DataTable(row["TABLE_NAME"] as string);

                            if (row["TABLE_SCHEMA"] != DBNull.Value &&
                                ((string)row["TABLE_SCHEMA"]) == connection.Database)
                            {
                                viewHash[view.TableName] = view;
                                information.Views.Add(view);
                            }
                        }
                    }
                    catch
                    {
                    }
                    #endregion

                    #region Columns
                    try
                    {
                        DataTable columns = connection.GetSchema("Columns");

                        foreach (DataRow row in columns.Rows)
                        {
                            if (row["TABLE_SCHEMA"] != DBNull.Value &&
                                ((string)row["TABLE_SCHEMA"]) != connection.Database) continue;

                            string columnName = row["COLUMN_NAME"] as string;
                            string tableName = row["TABLE_NAME"] as string;

                            Type columnType = ConvertDbTypeToTypeInternal(row["DATA_TYPE"] as string);

                            DataColumn column = new DataColumn(columnName, columnType);
                            DataTable table = tableHash[tableName] as DataTable;
                            DataTable view = viewHash[tableName] as DataTable;
                            if (table != null)
                            {
                                table.Columns.Add(column);
                            }
                            if (view != null)
                            {
                                view.Columns.Add(column);
                            }

                        }
                    }
                    catch
                    {
                    }
                    #endregion

                    #region Procedures
                    Hashtable procedureHash = new Hashtable();

                    try
                    {
                        DataTable procedures = connection.GetSchema("Procedures");

                        foreach (DataRow row in procedures.Rows)
                        {
                            if (row["ROUTINE_SCHEMA"] != DBNull.Value &&
                                ((string)row["ROUTINE_SCHEMA"]) == "sys") continue;

                            string procName = row["ROUTINE_NAME"] as string;
                            if (procName.IndexOf(";", StringComparison.InvariantCulture) != -1)
                                procName = procName.Substring(0, procName.IndexOf(";", StringComparison.InvariantCulture));

                            DataTable procedure = new DataTable(procName);

                            procedureHash[procedure.TableName] = procedure;
                            information.StoredProcedures.Add(procedure);
                        }
                    }
                    catch
                    {
                    }
                    #endregion

                    #region Procedures Parameters and Columns
                    foreach (DataTable procedures in information.StoredProcedures)
                    {
                        try
                        {
                            using (MySqlCommand myCommand = new MySqlCommand(procedures.TableName, connection))
                            {
                                myCommand.CommandType = CommandType.StoredProcedure;

                                MySqlCommandBuilder.DeriveParameters(myCommand);
                                MySqlDataReader reader = myCommand.ExecuteReader(CommandBehavior.SchemaOnly);
                                DataTable dataTable = new DataTable(procedures.TableName);
                                dataTable.Load(reader);

                                foreach (DataColumn column in dataTable.Columns)
                                {
                                    DataColumn procedureColumn = new DataColumn(column.ColumnName, column.DataType);
                                    procedureColumn.Caption = "Columns";
                                    procedures.Columns.Add(procedureColumn);
                                }

                                if (myCommand.Parameters.Count > 0)
                                {
                                    foreach (MySqlParameter param in myCommand.Parameters)
                                    {
                                        if (param.Direction == ParameterDirection.Input)
                                        {
                                            DataColumn procedureColumn = new DataColumn(param.ParameterName, ConvertDbTypeToTypeInternal(param.DbType.ToString()));
                                            procedureColumn.Caption = "Parameters";
                                            procedures.Columns.Add(procedureColumn);
                                        }
                                    }
                                }

                                reader.Dispose();
                            }
                        }
                        catch
                        {
                        }
                    }
                    #endregion

                    connection.Close();
                }

                return information;
            }
            catch
            {
                return null;
            }
        }

        protected Type ConvertDbTypeToTypeInternal(string dbType)
        {
            switch (dbType.ToLowerInvariant())
            {
                case "bigint":
                case "int":
                case "uniqueidentifier":
                case "smallint":
                case "tinyint":
                    return typeof(Int64);

                case "decimal":
                case "money":
                case "smallmoney":
                    return typeof(decimal);

                case "float":
                case "real":
                    return typeof(double);

                case "datetime":
                case "smalldatetime":
                case "timestamp":
                    return typeof(DateTime);

                default:
                    return typeof(string);
            }
        }


        public override DialogResult Edit(bool newDatabase)
        {
            using (StiSqlDatabaseEditForm form = new StiSqlDatabaseEditForm(this))
            {
                if (newDatabase) form.Text = StiLocalization.Get("FormDatabaseEdit", "MySQLNew");
                else form.Text = StiLocalization.Get("FormDatabaseEdit", "MySQLEdit");

                form.tbName.Text = this.Name;
                form.tbAlias.Text = this.Alias;
                form.tbConnectionString.Text = this.ConnectionString;
                if (form.ShowDialog() == DialogResult.OK)
                {
                    this.Name = form.tbName.Text;
                    this.Alias = form.tbAlias.Text;
                    this.ConnectionString = form.tbConnectionString.Text;
                }
                return form.DialogResult;
            }
        }
        #endregion

        /// <summary>
        /// Creates a new object of the type StiMySqlDatabase.
        /// </summary>
        public StiMySqlDatabase()
            : this(string.Empty, string.Empty)
        {
        }


        /// <summary>
        /// Creates a new object of the type StiMySqlDatabase.
        /// </summary>
        public StiMySqlDatabase(string name, string connectionString)
            : base(name, connectionString)
        {
        }


        /// <summary>
        /// Creates a new object of the type StiMySqlDatabase.
        /// </summary>
        public StiMySqlDatabase(string name, string alias, string connectionString)
            : base(name, alias, connectionString)
        {
        }


        /// <summary>
        /// Creates a new object of the type StiMySqlDatabase.
        /// </summary>
        public StiMySqlDatabase(string name, string alias, string connectionString, bool promptUserNameAndpassword)
            :
            base(name, alias, connectionString, promptUserNameAndpassword)
        {
        }

        /// <summary>
        /// Creates a new object of the type StiMySqlDatabase.
        /// </summary>
        public StiMySqlDatabase(string name, string alias, string connectionString, bool promptUserNameAndpassword, string key)
            :
            base(name, alias, connectionString, promptUserNameAndpassword, key)
        {
        }

    }
}
